#!/usr/bin/env python 

from boundedValueClass import *
from languageClass import *

from xml.dom.ext.reader.Sax import FromXmlFile 
from xml.dom import Node

from dom_tools import *

class Movie:

    def __init__(self):
      self.reference = ""
      self.title = ""
      self.language = Language("")
      self.subtitles = []
      self.rating = BoundedValue(aValue=0, aMin=0,aMax=5)
      self.quality = BoundedValue(aValue=0, aMin=0,aMax=5)
      self.genre = ""

    def read_from_file(self,aFilename):
      doc = FromXmlFile(aFilename)
      if len(doc.getElementsByTagName("movie-info")) == 1:
        return self.read_from_xml_tag(doc.getElementsByTagName("movie-info")[0])
      else: 
        return None
    read_from_file = classmethod(read_from_file)

    def read_from_xml_tag(self, anXMLTag):
      anInstance = Movie()
      anInstance.title = anXMLTag.getAttribute("title")
      anInstance.reference = anXMLTag.getAttribute("reference")

      ## get language
      languageNode = find_nodeName(anXMLTag, "language")
      if languageNode is not None :
        anInstance.language = Language.read_from_XML_tag(languageNode)
			
      ## get rating
      ratingNode = find_nodeName(anXMLTag,"rating")
      if ratingNode is not None :
        anInstance.rating = BoundedValue.read_from_XML_tag(ratingNode)

      ## get quality
      qualityNode = find_nodeName(anXMLTag,"quality")
      if qualityNode is not None :
        anInstance.quality = BoundedValue.read_from_XML_tag(qualityNode)

      ## get subtuitles
      subtitlesNode = find_nodeName(anXMLTag,"subtitles")
      if subtitlesNode is not None :
        subtitles = []
        for aSubtitleTag in find_all_nodeName(subtitlesNode,"language"):
          subtitles.append(Subtitle.read_from_XML_tag(aSubtitleTag))
          anInstance.subtitles = subtitles

      ## get genre
      genreNode = find_nodeName(anXMLTag,"genre")
      if genreNode is not None :
        anInstance.genre = genreNode.getAttribute("name")

      return anInstance
    read_from_xml_tag = classmethod(read_from_xml_tag)


    def save_as(self, aFilename):
      file = open(aFilename, w)
      file.write("""<?xml version="1.0"?>
<!-- WARNING: this xml is case sensitive! --> 
%s""" % self.to_xml() )

    def to_xml(self, leftOffset = 0):
      offsetString = leftOffset * (' ')
      xml = '%(offset)s<movie-info'% {'offset':offsetString}
      if self.reference is not None:
        xml += ' reference="%s"' % self.reference
      xml += ' title="%s">' % self.title
      
      if self.language is not None:
        xml += "\n%(offset)s  %(value)s"  % {'offset':offsetString,'value':self.language.to_xml()}
      else:
        xml += "\n%(offset)s  <!-- no language -->" % {'offset':offsetString}
      if self.rating is not None:
        xml += "\n%(offset)s  %(value)s" % {'offset':offsetString,'value':self.rating.to_xml("rating")}
      if self.quality is not None:
        xml += "\n%(offset)s  %(value)s" % {'offset':offsetString,'value':self.quality.to_xml("quality")}
      if len( self.subtitles) > 0:
        xml += "\n%(offset)s  <subtitles>" % {'offset':offsetString}
        for aSubtitle in self.subtitles:
          xml += "\n%(offset)s    %(value)s" % {'offset':offsetString,'value':aSubtitle.to_xml()}
          xml += "\n%(offset)s  </subtitles>" % {'offset':offsetString}

      if self.genre is not None:
          xml += "\n%(offset)s  <genre value=\"%(value)s\" />"  % {'offset':offsetString,'value': self.genre}
      else:
          xml += "\n%(offset)s <!-- no genre -->" % {'offset':offsetString}

      xml += "\n%(offset)s</movie-info>" % {'offset':offsetString}

      return xml

    def merge_with(self, aMovie):
        self.reference = aMovie.reference
        self.title = aMovie.title
        self.language = aMovie.language
        self.subtitles = aMovie.subtitles
        self.rating.merge_with(aMovie.rating)
        self.quality.merge_with(aMovie.quality)
        self.genre = aMovie.genre

    def subtitles_codes(self):
        allCodes = []
        for aLanguage in self.subtitles:
            allCodes.append(str(aLanguage.code))
        return allCodes
        
    def __str__(self):
        return """[Movie]
  reference: #%s
  title: "%s"
  language: %s
  subtitles: %s
  rating: %s
  quality: %s
  genre: %s
""" % (self.reference,
       self.title,
       self.language.code,
       self.subtitles_codes(),
       self.rating.value,
       self.quality.value,
       self.genre)
